from typing import Optional, Dict

from requests.sessions import Session
from requests.models import Response

from fishtext.types import TextType, TextFormat, JsonAPIResponse
from fishtext.errors import (
    TooManyContentExceeded,
    CallLimitExceeded,
    BannedForever,
    InternalServerError,
)


FISH_TEXT_API_URL = "https://fish-text.ru/get"
FISH_TEXT_API_DOCS = "https://fish-text.ru/api"


class FishTextAPI:
    def __init__(
        self, *,
        api_url: str,
        text_type: TextType,
        text_format: TextFormat,
        session: Optional[Session] = None,
    ):
        self.session = session or Session()
        self.api_url = api_url
        self.text_type = text_type
        self.text_format = text_format

    def process_response(self, response) -> None:
        raise NotImplementedError

    def get(self, number: int = 100):
        raise NotImplementedError


class FishTextJson(FishTextAPI):
    def __init__(
        self, *,
        session: Optional[Session] = None,
        api_url: str = FISH_TEXT_API_URL,
        text_type: TextType = TextType.Sentence,
    ):
        super().__init__(
            session=session,
            api_url=api_url,
            text_type=text_type,
            text_format=TextFormat.json,
        )

    def process_response(self, response: JsonAPIResponse) -> None:

        if response.errorCode is None:
            return

        error_codes = {
            11: TooManyContentExceeded,
            21: CallLimitExceeded,
            22: BannedForever,
            31: InternalServerError,
        }

        exception = error_codes.get(response.errorCode, None)
        if exception:
            raise exception(response.text)

    def get(self, number: int = 100) -> JsonAPIResponse:
        json_response = self.session.get(
            FISH_TEXT_API_URL,
            params=dict(
                format=self.text_format, number=number, type=self.text_type
            ),
        ).json()
        json_api_response_object = JsonAPIResponse(
            status=json_response.get("status"),
            text=json_response.get("text"),
            errorCode=json_response.get("errorCode", None),
        )
        self.process_response(json_api_response_object)
        return json_api_response_object


class FishTextHtml(FishTextAPI):

    TOO_MUCH_CONTENT_EXCEEDED = (
        "You requested too much content. Be more moderate."
    )

    def __init__(
        self, *,
        session: Optional[Session] = None,
        api_url: str = FISH_TEXT_API_URL,
        text_type: TextType = TextType.Sentence,
    ):
        super().__init__(
            session=session,
            api_url=api_url,
            text_type=text_type,
            text_format=TextFormat.html,
        )

    def process_response(self, response: Response) -> None:
        if response.text == self.TOO_MUCH_CONTENT_EXCEEDED:
            raise TooManyContentExceeded(response.text)
        if response.status_code == 403:
            raise CallLimitExceeded(response.text)
            # TODO: дописать BannedForever
        if response.status_code == 500:
            raise InternalServerError(response.text)

    def get(self, number: int = 100) -> str:
        response = self.session.get(
            FISH_TEXT_API_URL,
            params=dict(
                format=self.text_format, number=number, type=self.text_type
            ),
        )
        self.process_response(response)
        return response.text



class TooManyContentExceeded(Exception):
    def __init__(self, message):
        super().__init__(message)


class CallLimitExceeded(Exception):
    def __init__(self, message):
        super().__init__(message)


class BannedForever(Exception):
    def __init__(self, message):
        super().__init__(message)


class InternalServerError(Exception):
    def __init__(self, message):
        super().__init__(message)



from typing import Optional
from dataclasses import dataclass
from enum import Enum


class TextType(str, Enum):
    Sentence = "sentence"
    Paragraph = "paragraph"
    Title = "title"


class TextFormat(str, Enum):
    json = "json"
    html = "html"


@dataclass
class JsonAPIResponse:
    status: str
    text: str
    errorCode: Optional[int]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fishtext']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'fish-text-ru',
    'version': '2.0.1',
    'description': 'Simple fish-text.ru python wrapper',
    'long_description': '<h1 align="center">Fish-text.ru python wrapper</h1>\n\nЭта небольшая обёртка позволяет использовать API с сайта Fish-text.ru для генерации так называемого "текста-рыбы" или "рыботекста"\n\nТестировано на Python 3.8, 3.9.1\n\n## Примеры использования\n* В вашем проекте необходимо заполнить базу данных каким-то текстовым контентом\n* Собственно, наверное, это все ¯\\_(ツ)_/¯\n\n## Установка\nУстановить стабильную версию с PyPi:\n```\npython -m pip install fish_text_ru \n```\nУстановить с GitHub:\n```\npython -m pip install git+https://github.com/kiriharu/fish_text_ru\n```\n\n## Использование\n### Json Wrapper (рекомендуется)\n\n```python\n# Импортируем FishTextJson и TextType, нужный нам.\n# В методе .get() вернется по итогу объект JsonAPIResponse\nfrom fishtext import FishTextJson\nfrom fishtext.types import TextType, JsonAPIResponse\n\n# Делаем объект\napi = FishTextJson(text_type=TextType.Title)\n# Используем!\ntitles = api.get(1)\n\n# Используем JsonAPIResponse\nprint(titles.status)\nprint(titles.errorCode)\nprint(titles.text)\n```\n\n### Html wrapper\n```python\n\nfrom fishtext import FishTextHtml\nfrom fishtext.types import TextType\n\n# Делаем объект\napi = FishTextHtml(text_type=TextType.Title)\n# Используем!\ntitle = api.get(1)\n\nprint(title) # <p> какой-то там title </p>\n\n```',
    'author': 'kiriharu',
    'author_email': 'kiriharu@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kiriharu/fish_text_ru',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
